<!--
MIT License

Copyright (c) 2021 Bootcamp-Project contributors <contributors@bootcamp-project.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
-->

# Ready-to-use TypeScript Configuration

```json
{
	"compilerOptions": {
		/* Visit https://aka.ms/tsconfig.json to read more about this file */

		/* Projects */
		// "incremental": true,								/* Enable incremental compilation */
		// "composite": true,								/* Enable constraints that allow a TypeScript project to be used with project references. */
		// "tsBuildInfoFile": "./",							/* Specify the folder for .tsbuildinfo incremental compilation files. */
		// "disableSourceOfProjectReferenceRedirect": true,	/* Disable preferring source files instead of declaration files when referencing composite projects */
		// "disableSolutionSearching": true,				/* Opt a project out of multi-project reference checking when editing. */
		// "disableReferencedProjectLoad": true,			/* Reduce the number of projects loaded automatically by TypeScript. */

		/* Language and Environment */
		// USE ONLY LOCAL! // "target": "es5",	/* Set the JavaScript language version for emitted JavaScript and include compatible library declarations. */
		// USE ONLY LOCAL! // "lib": [],		/* Specify a set of bundled library declaration files that describe the target runtime environment. */
		// "jsx": "preserve",					/* Specify what JSX code is generated. */
		// "experimentalDecorators": true,		/* Enable experimental support for TC39 stage 2 draft decorators. */
		// "emitDecoratorMetadata": true,		/* Emit design-type metadata for decorated declarations in source files. */
		// "jsxFactory": "",					/* Specify the JSX factory function used when targeting React JSX emit, e.g. 'React.createElement' or 'h' */
		// "jsxFragmentFactory": "",			/* Specify the JSX Fragment reference used for fragments when targeting React JSX emit e.g. 'React.Fragment' or 'Fragment'. */
		// "jsxImportSource": "",				/* Specify module specifier used to import the JSX factory functions when using `jsx: react-jsx*`.` */
		// "reactNamespace": "",				/* Specify the object invoked for `createElement`. This only applies when targeting `react` JSX emit. */
		// "noLib": true,						/* Disable including any library files, including the default lib.d.ts. */
		// "useDefineForClassFields": true,		/* Emit ECMAScript-standard-compliant class fields. */

		/* Modules */
		"moduleResolution": "node",					/* Specify how TypeScript looks up a file from a given module specifier. */
		// USE ONLY LOCAL! "module": "commonjs",	/* Specify what module code is generated. */
		// USE ONLY LOCAL! // "baseUrl": "./",		/* Specify the base directory to resolve non-relative module names. */
		// USE ONLY LOCAL! // "rootDir": "./",		/* Specify the root folder within your source files. */
		// USE ONLY LOCAL! // "paths": {},			/* Specify a set of entries that re-map imports to additional lookup locations. */
		// USE ONLY LOCAL! // "rootDirs": [],		/* Allow multiple folders to be treated as one when resolving modules. */
		// USE ONLY LOCAL! // "typeRoots": [],		/* Specify multiple folders that act like `./node_modules/@types`. */
		// USE ONLY LOCAL! // "types": [],			/* Specify type package names to be included without being referenced in a source file. */
		"allowUmdGlobalAccess": true,				/* Allow accessing UMD globals from modules. */
		"resolveJsonModule": true,					/* Enable importing .json files */
		// "noResolve": true,						/* Disallow `import`s, `require`s or `<reference>`s from expanding the number of files TypeScript should add to a project. */

		/* JavaScript Support */
		"allowJs": true,				/* Allow JavaScript files to be a part of your program. Use the `checkJS` option to get errors from these files. */
		"checkJs": true,				/* Enable error reporting in type-checked JavaScript files. */
		// "maxNodeModuleJsDepth": 1,	/* Specify the maximum folder depth used for checking JavaScript files from `node_modules`. Only applicable with `allowJs`. */

		/* Emit */
		// USE ONLY LOCAL! // "outFile": "./",	/* Specify a file that bundles all outputs into one JavaScript file. If `declaration` is true, also designates a file that bundles all .d.ts output. */
		// USE ONLY LOCAL! // "outDir": "./",	/* Specify an output folder for all emitted files. */
		"removeComments": true,					/* Disable emitting comments. */
		"emitBOM": true,						/* Emit a UTF-8 Byte Order Mark (BOM) in the beginning of output files. */
		"newLine": "lf",						/* Set the newline character for emitting files. */
		// USE ONLY LOCAL! // "noEmit": true,	/* Disable emitting files from a compilation. */
		// "noEmitOnError": true,				/* Disable emitting files if any type checking errors are reported. */

		"declaration": true,								/* Generate .d.ts files from TypeScript and JavaScript files in your project. */
		"declarationMap": true,								/* Create sourcemaps for d.ts files. */
		// USE ONLY LOCAL! // "emitDeclarationOnly": true,	/* Only output d.ts files and not JavaScript files. */
		// "declarationDir": "./",							/* Specify the output directory for generated declaration files. */

		"sourceMap": true,								/* Create source map files for emitted JavaScript files. */
		// USE ONLY LOCAL! // "mapRoot": "",			/* Specify the location where debugger should locate map files instead of generated locations. */
		// USE ONLY LOCAL! // "sourceRoot": "",			/* Specify the root path for debuggers to find the reference source code. */
		// USE ONLY LOCAL! // "inlineSourceMap": true,	/* Include sourcemap files inside the emitted JavaScript. */
		// USE ONLY LOCAL! // "inlineSources": true,	/* Include source code in the sourcemaps inside the emitted JavaScript. */

		"downlevelIteration": true,						/* Emit more compliant, but verbose and less performant JavaScript for iteration. */
		// USE ONLY LOCAL! // "importHelpers": true,	/* Allow importing helper functions from tslib once per project, instead of including them per-file. */
		// "noEmitHelpers": true,						/* Disable generating custom helper functions like `__extends` in compiled output. */
		// "importsNotUsedAsValues": "remove",			/* Specify emit/checking behavior for imports that are only used for types */
		// "stripInternal": true,						/* Disable emitting declarations that have `@internal` in their JSDoc comments. */
		// "preserveConstEnums": true,					/* Disable erasing `const enum` declarations in generated code. */

		/* Interop Constraints */
		"isolatedModules": true,					/* Ensure that each file can be safely transpiled without relying on other imports. */
		"allowSyntheticDefaultImports": true,		/* Allow 'import x from y' when a module doesn't have a default export. */
		"esModuleInterop": true,					/* Emit additional JavaScript to ease support for importing CommonJS modules. This enables `allowSyntheticDefaultImports` for type compatibility. */
		"preserveSymlinks": true,					/* Disable resolving symlinks to their realpath. This correlates to the same flag in node. */
		"forceConsistentCasingInFileNames": true,	/* Ensure that casing is correct in imports. */

		/* Type Checking */
		"alwaysStrict": true,					/* Ensure 'use strict' is always emitted. */
		"strict": true,							/* Enable all strict type-checking options. */
		"strictNullChecks": true,				/* When type checking, take into account `null` and `undefined`. */
		"strictFunctionTypes": true,			/* When assigning functions, check to ensure parameters and the return values are subtype-compatible. */
		"strictBindCallApply": true,			/* Check that the arguments for `bind`, `call`, and `apply` methods match the original function. */
		"strictPropertyInitialization": true,	/* Check for class properties that are declared but not set in the constructor. */

		"noImplicitAny": true,						/* Enable error reporting for expressions and declarations with an implied `any` type.. */
		"noImplicitThis": true,						/* Enable error reporting when `this` is given the type `any`. */
		"noUnusedLocals": true,						/* Enable error reporting when a local variables aren't read. */
		"noUnusedParameters": true,					/* Raise an error when a function parameter isn't read */
		"noImplicitReturns": true,					/* Enable error reporting for codepaths that do not explicitly return in a function. */
		"noFallthroughCasesInSwitch": true,			/* Enable error reporting for fallthrough cases in switch statements. */
		"noUncheckedIndexedAccess": true,			/* Include 'undefined' in index signature results */
		"noImplicitOverride": true,					/* Ensure overriding members in derived classes are marked with an override modifier. */
		"noPropertyAccessFromIndexSignature": true,	/* Enforces using indexed accessors for keys declared using an indexed type */

		"useUnknownInCatchVariables": true,	/* Type catch clause variables as 'unknown' instead of 'any'. */

		// "allowUnusedLabels": true,		/* Disable error reporting for unused labels. */
		// "allowUnreachableCode": true,	/* Disable error reporting for unreachable code. */

		// "exactOptionalPropertyTypes": true,	/* Interpret optional property types as written, rather than adding 'undefined'. */

		/* Completeness */
		"skipDefaultLibCheck": true,	/* Skip type checking .d.ts files that are included with TypeScript. */
		"skipLibCheck": true,			/* Skip type checking all .d.ts files. */
		"pretty": true,
	},
	"compileOnSave": false,
	"include": [
		"src/**/*"
	],
	"exclude": [
		"node_modules/*"
	]
}
```

```ts
import _ from 'lodash'
import { ESLintConfig, ESLintSecurity } from '@bootcamp-project/eslinttier'

export const ESLintConfigTypeScript = _.merge(ESLintConfig, {
	// Specifies the ESLint parser
	parser: '@typescript-eslint/parser',
	plugins: ['@typescript-eslint/eslint-plugin'],
	extends: ['plugin:@typescript-eslint/recommended', 'plugin:@typescript-eslint/recommended-requiring-type-checking'],
	parserOptions: {
		tsconfigRootDir: './',
		project: ['./tsconfig.json']
	}
})

export const ESLintSecurityTypeScript = _.merge(ESLintSecurity, {
	// Specifies the ESLint parser
	parser: '@typescript-eslint/parser',
	plugins: ['@typescript-eslint/eslint-plugin'],
	extends: ['plugin:@typescript-eslint/recommended', 'plugin:@typescript-eslint/recommended-requiring-type-checking'],
	parserOptions: {
		tsconfigRootDir: './',
		project: ['./tsconfig.json']
	}
})
```

```ts
import _ from 'lodash'
import { ESLintConfigTypeScript, ESLintSecurityTypeScript } from './typescript.eslint'

export const ESLintConfigTSDoc = _.merge(ESLintConfigTypeScript, {
	plugins: ['eslint-plugin-tsdoc'],
	rules: {
		// Place to specify ESLint rules. Can be used to overwrite rules specified from the extended configs
		'tsdoc/syntax': 'warn'
	}
})
export const ESLintSecurityTSDoc = _.merge(ESLintSecurityTypeScript, {
	plugins: ['eslint-plugin-tsdoc'],
	rules: {
		// Place to specify ESLint rules. Can be used to overwrite rules specified from the extended configs
		'tsdoc/syntax': 'warn'
	}
})
```
